import argparse
from payloadinspector import get_field_values, BitField, Payload, format_output

def main():
    desc = 'Map a data payload to a specified bitfield'

    argparser = argparse.ArgumentParser(description=desc, formatter_class=argparse.RawTextHelpFormatter)
    input_group = argparser.add_mutually_exclusive_group()
    input_group.add_argument('-b', '--bitfield', help='Bitfield format as a list of tuples with a name-string and number of bits. Allowed formats:\n'+
                                                      '"(field1, 1), (field2, 2), (field3, 3), (field4, 4)"\n'+
                                                      '"(\'field1\', 1), (\'field2\', 2), (\'field3\', 3), (\'field4\', 4)"\n'+
                                                      '\'("field1", 1), ("field2", 2), ("field3", 3), ("field4", 4)\'\n'+
                                                      '"(\'field1\', 1),(\'field2\', 2),(\'field3\', 3),(\'field4\', 4)"\n'+
                                                      '"[(\'field1\', 1), (\'field2\', 2), (\'field3\', 3), (\'field4\', 4)]"')
    input_group.add_argument('-s', '--source-struct', help='Bitfield read as a struct from a C source file (or header).'+
                                                           'The format is "<file-path>:<structname>"')
    argparser.add_argument('-p', '--payload', help='The data to be put into the bitfield. Allowed formats:\n'+
                                                    '"0x0a 0x80 0xff"\n'+
                                                    '"0a 80 ff"\n'+
                                                    '"0x0a, 0x80, 0xff"\n'+
                                                    '"0a, 80, ff"\n'+
                                                    '"[0x0a, 0x80, 0xff]"\n'+
                                                    '"[0a, 80, ff]"\n'+
                                                    '"0x0a0x800xff"\n'+
                                                    '"0a80ff"'+
                                                    'If this argument is omitted, the bitfield format is printed.')
    argparser.add_argument('-f', '--format', help='Output format specifier. By default all bitfield elements are printed as decimal values.\n'+
                                                  'The format string can specify \'bin\', \'hex\' or \'dec\' to force a different format for all values.\n'+
                                                  'Alternatively, to only show some elements, a list of element names can be given, ex: \'field1, field3\'.\n'+
                                                  'It is also possible to specify different encoding formats for different elements, ex: \'field1:bin, field3:hex\'',
                                             default='dec')
    argparser.add_argument('--version', action='version', version='%(prog)s 1.0.1')
    args = argparser.parse_args()
    if args.bitfield:
        try:
            bf = BitField.from_string(args.bitfield)
        except Exception as err:
            print(f'ERROR: {err}.')
            exit(1)
    elif args.source_struct:
        try:
            file_name, struct_name = args.source_struct.split(':')
            bf = BitField.from_c_source(file_name, struct_name)
        except Exception as err:
            print(f'ERROR: {err}.')
            exit(1)
    else:
        print("ERROR: Either -b or -s argument must be given")
        exit(1)

    if args.payload:
        try:
            payload = Payload.from_string(args.payload)
        except Exception as err:
            print(f'ERROR: {err}.')
            exit(1)
    else:
        print(bf)
        exit(0)

    if bf.total_bits > payload.total_bits:
        print(f'ERROR: Bitfield is {bf.total_bits} bits, but payload is only {payload.total_bits} bits.')
        exit(1)
    if bf.total_bits < payload.total_bits:
        print(f'WARNING: Bitfield is {bf.total_bits} bits, and payload is {payload.total_bits} bits. Extra payload data is discarded.')
    try:
        output_string = format_output(get_field_values(bf, payload), args.format)
    except Exception as err:
        print(f'ERROR: {err}.')
        exit(1)
    print(output_string)

if __name__ == '__main__':
    main()



import pytest
from payloadinspector import BitField, BitFieldElement

def test_init_with_bitfieldelements():
    elements = [BitFieldElement('field1', 1),
                BitFieldElement('field2', 2),
                BitFieldElement('field3', 3),
                BitFieldElement('field4', 4)]
    bitfield = BitField(elements)

def test_init_with_tuples():
    elements = [('field1', 1),
                ('field2', 2),
                ('field3', 3),
                ('field4', 4)]
    bitfield = BitField(elements)
    for i in range(len(elements)):
        assert isinstance(bitfield[i], BitFieldElement)

def test_init_with_mixed_args():
    elements = [('field1', 1),
                BitFieldElement('field2', 2),
                ('field3', 3),
                BitFieldElement('field4', 4)]
    bitfield = BitField(elements)
    for i in range(len(elements)):
        assert isinstance(bitfield[i], BitFieldElement)

def test_init_with_string_n_bits():
    elements = [('field1', 'string')]
    with pytest.raises(TypeError):
        bitfield = BitField(elements)

def test_init_with_float_n_bits():
    elements = [('field1', 1.2)]
    with pytest.raises(TypeError):
        bitfield = BitField(elements)

def test_init_with_int_name():
    elements = [(1, 1)]
    with pytest.raises(TypeError):
        bitfield = BitField(elements)

def test_init_with_duplicate_names():
    elements = [('field1', 1), ('field1', 1)]
    with pytest.raises(ValueError):
        bitfield = BitField(elements)

def test_iteration():
    elements = [('field1', 1),
                ('field2', 2),
                ('field3', 3),
                ('field4', 4)]
    bitfield = BitField(elements)
    assert [n.name for n in bitfield] == ['field1', 'field2', 'field3', 'field4']
    assert [n.n_bits for n in bitfield] == [1, 2, 3, 4]

def test_subscripting():
    elements = [('field1', 1),
                ('field2', 2),
                ('field3', 3),
                ('field4', 4)]
    bitfield = BitField(elements)
    for i in range(4):
        assert bitfield[i] == BitFieldElement(f'field{i+1}', i+1)

def test_equal():
    elements1 = [('field1', 1),
                 ('field2', 2),
                 ('field3', 3),
                 ('field4', 4)]
    elements2 = [BitFieldElement('field1', 1),
                 BitFieldElement('field2', 2),
                 BitFieldElement('field3', 3),
                 BitFieldElement('field4', 4)]
    bf1 = BitField(elements1)
    bf2 = BitField(elements2)
    assert bf1 == bf2

def test_unequal_sizes():
    elements1 = [('field1', 1),
                 ('field2', 2),
                 ('field3', 3),
                 ('field4', 4)]
    elements2 = [BitFieldElement('field1', 1),
                 BitFieldElement('field2', 2),
                 BitFieldElement('field3', 3)]
    bf1 = BitField(elements1)
    bf2 = BitField(elements2)
    assert bf1 != bf2

def test_unequal_bits():
    elements1 = [('field1', 1),
                 ('field2', 2),
                 ('field3', 3),
                 ('field4', 4)]
    elements2 = [BitFieldElement('field1', 100),
                 BitFieldElement('field2', 2),
                 BitFieldElement('field3', 3),
                 BitFieldElement('field4', 4)]
    bf1 = BitField(elements1)
    bf2 = BitField(elements2)
    assert bf1 != bf2

def test_unequal_names():
    elements1 = [('field1', 1),
                 ('field2', 2),
                 ('field3', 3),
                 ('field4', 4)]
    elements2 = [BitFieldElement('field_one', 1),
                 BitFieldElement('field2', 2),
                 BitFieldElement('field3', 3),
                 BitFieldElement('field4', 4)]
    bf1 = BitField(elements1)
    bf2 = BitField(elements2)
    assert bf1 != bf2

def test_len():
    elements = [('field1', 1),
                ('field2', 2),
                ('field3', 3),
                ('field4', 4)]
    bitfield = BitField(elements)
    assert len(bitfield) == 4

def test_total_bits():
    elements = [('field1', 1),
                ('field2', 2),
                ('field3', 3),
                ('field4', 4)]
    bitfield = BitField(elements)
    assert bitfield.total_bits == 10

def string_initializer_test(string: str):
    elements = [('field1', 1),
                ('field2', 2),
                ('field3', 3),
                ('field4', 4)]
    bitfield_from_list = BitField(elements)
    bitfield_from_string = BitField.from_string(string)

    assert bitfield_from_list == bitfield_from_string

def test_string_initialization_single_quotes():
    string = "('field1', 1), ('field2', 2), ('field3', 3), ('field4', 4)"
    string_initializer_test(string)

def test_string_initialization_double_quotes():
    string = '("field1", 1), ("field2", 2), ("field3", 3), ("field4", 4)'
    string_initializer_test(string)

def test_string_initialization_no_quotes():
    string = '(field1, 1), (field2, 2), (field3, 3), (field4, 4)'
    string_initializer_test(string)

def test_string_initialization_without_whitespace():
    string = "('field1', 1),('field2', 2),('field3', 3),('field4', 4)"
    string_initializer_test(string)

def test_string_initialization_with_square_brackets():
    string = "[('field1', 1), ('field2', 2), ('field3', 3), ('field4', 4)]"
    string_initializer_test(string)

def test_string_initialization_with_newlines():
    string = "[('field1', 1),\n('field2', 2), \n('field3', 3),\n ('field4', 4)]"
    string_initializer_test(string)

def assert_equal_bitfields(bf1: BitField, bf2: BitField):
    '''Helper function to find which elements if any dont match'''
    assert len(bf1) == len(bf2)
    for i in range(len(bf1)):
        assert bf1[i].name == bf2[i].name
        assert bf1[i].n_bits == bf2[i].n_bits

def test_header_file_message():
    bitfield_from_file = BitField.from_c_source('tests/header.h', 'message')
    true_bitfield = BitField([('field1', 1), ('field2', 2), ('big_value', 32), ('field3', 3)])
    assert_equal_bitfields(bitfield_from_file, true_bitfield)

def test_header_file_message2():
    bitfield_from_file = BitField.from_c_source('tests/header.h', 'message2')
    true_bitfield = BitField([('field4', 4), ('field5', 5)])
    assert_equal_bitfields(bitfield_from_file, true_bitfield)

def test_header_file_message3():
    bitfield_from_file = BitField.from_c_source('tests/header.h', 'message3')
    true_bitfield = BitField([('field6', 6), ('field7', 7)])
    assert_equal_bitfields(bitfield_from_file, true_bitfield)

def test_header_file_message4():
    bitfield_from_file = BitField.from_c_source('tests/header.h', 'message4')
    true_bitfield = BitField([('field8', 8), ('field9', 9)])
    assert_equal_bitfields(bitfield_from_file, true_bitfield)

def test_header_file_message5():
    bitfield_from_file = BitField.from_c_source('tests/header.h', 'message5')
    true_bitfield = BitField([('field1', 1), ('field2', 2)])
    assert_equal_bitfields(bitfield_from_file, true_bitfield)

def test_header_file_message6():
    bitfield_from_file = BitField.from_c_source('tests/header.h', 'message6')
    true_bitfield = BitField([('field1', 1), ('field2', 2)])
    assert_equal_bitfields(bitfield_from_file, true_bitfield)

def test_missing_struct():
    with pytest.raises(ValueError):
        BitField.from_c_source('tests/header.h', 'message_does_not_exist')

def test_header_file_packed_struct():
    bitfield_from_file = BitField.from_c_source('tests/header.h', 'packed_struct')
    true_bitfield = BitField([('field1', 1), ('field2', 2)])
    assert_equal_bitfields(bitfield_from_file, true_bitfield)

def test_header_file_packed_struct_wo_whitespace():
    bitfield_from_file = BitField.from_c_source('tests/header.h', 'packed_struct_wo_whitespace')
    true_bitfield = BitField([('field1', 1), ('field2', 2)])
    assert_equal_bitfields(bitfield_from_file, true_bitfield)

def test_header_file_missing_bit_number():
    with pytest.raises(ValueError):
        BitField.from_c_source('tests/header.h', 'struct_with_missing_bit_number')

def test_header_file_missing_bit_number2():
    with pytest.raises(ValueError, match='Some elements are missing bit size'):
        BitField.from_c_source('tests/header.h', 'struct_with_missing_bit_number2')

def test_header_file_defined_bit_number():
    with pytest.raises(ValueError, match='Could not parse bit size.*'):
        BitField.from_c_source('tests/header.h', 'struct_with_defined_bit_number')



import pytest
from payloadinspector import format_output

def test_no_format():
    result = {'field1': 1, 'field2': 128, 'field3': 215}
    output = format_output(result)
    assert output == 'field1: 1, field2: 128, field3: 215'

def test_dec_format():
    result = {'field1': 1, 'field2': 128, 'field3': 215}
    output = format_output(result, 'dec')
    assert output == 'field1: 1, field2: 128, field3: 215'

def test_hex_format():
    result = {'field1': 1, 'field2': 128, 'field3': 215}
    output = format_output(result, 'hex')
    assert output == 'field1: 0x1, field2: 0x80, field3: 0xd7'

def test_bin_format():
    result = {'field1': 1, 'field2': 128, 'field3': 215}
    output = format_output(result, 'bin')
    assert output == 'field1: 0b1, field2: 0b10000000, field3: 0b11010111'

def test_one_element():
    result = {'field1': 1, 'field2': 128, 'field3': 215}
    output = format_output(result, 'field2')
    assert output == 'field2: 128'

def test_two_elements():
    result = {'field1': 1, 'field2': 128, 'field3': 215}
    output = format_output(result, 'field2, field3')
    assert output == 'field2: 128, field3: 215'

def test_different_formatting():
    result = {'field1': 1, 'field2': 128, 'field3': 215}
    output = format_output(result, 'field1: dec, field2:bin, field3:hex')
    assert output == 'field1: 1, field2: 0b10000000, field3: 0xd7'

def test_one_element_with_formatting():
    result = {'field1': 1, 'field2': 128, 'field3': 215}
    output = format_output(result, 'field2:bin')
    assert output == 'field2: 0b10000000'

def test_two_elements_with_formatting():
    result = {'field1': 1, 'field2': 128, 'field3': 215}
    output = format_output(result, 'field2:bin, field3:hex')
    assert output == 'field2: 0b10000000, field3: 0xd7'

def test_nonexisting_element():
    result = {'field1': 1, 'field2': 128, 'field3': 215}
    with pytest.raises(ValueError):
        format_output(result, 'field4')



from payloadinspector import get_bits_from_byte

def test_1():
    byte = 0b01010101
    bits = get_bits_from_byte(byte, 1, 3)
    assert bits == 0b101

def test_2():
    byte = 0b00110000
    bits = get_bits_from_byte(byte, 1, 3)
    assert bits == 0b011

def test_3():
    byte = 0b00110000
    bits = get_bits_from_byte(byte, 1, 4)
    assert bits == 0b0110

def test_4():
    byte = 0b00110011
    bits = get_bits_from_byte(byte, 0, 8)
    assert bits == 0b00110011

def test_5():
    byte = 0b00110011
    bits = get_bits_from_byte(byte, 4, 4)
    assert bits == 0b0011

def test_6():
    byte = 0b11111111
    bits = get_bits_from_byte(byte, 0, 8)
    assert bits == 0b11111111

def test_7():
    byte = 0b11111111
    bits = get_bits_from_byte(byte, 0, 0)
    assert bits == 0b0

def test_8():
    byte = 0b11111111
    bits = get_bits_from_byte(byte, 0, 1)
    assert bits == 0b1



from payloadinspector import get_field_values, BitField, Payload

class Case:
    def __init__(self, bitfield: BitField, payload: Payload, correct_result: dict[str, int]):
        self.bitfield = bitfield
        self.payload = payload
        self.correct_result = correct_result


def test_case_1():
    # 0001 0010 1010 1100
    # 000 10010 101011 00
    #   0    18     43  0
    run_test_case(Case(BitField([('field1', 3), ('field2', 5), ('field3', 6), ('field4', 2)]),
                       Payload([0x12, 0xac]),
                       {'field1': 0, 'field2': 18, 'field3': 43, 'field4': 0}))

def test_case_2():
    # 0001 0010 1010 1100
    # 000100101 0101 100
    #        37     5   4
    run_test_case(Case(BitField([('field1', 9), ('field2', 4), ('field3', 3)]),
                       Payload([0x12, 0xac]),
                       {'field1': 37, 'field2': 5, 'field3': 4}))

def test_case_3():
    # 0001 0010 1010 1100
    # 000 10010101011 00
    #   0        1195  0
    run_test_case(Case(BitField([('field1', 3), ('field2', 11), ('field3', 2)]),
                       Payload([0x12, 0xac]),
                       {'field1': 0, 'field2': 1195, 'field3': 0}))


def run_test_case(test: Case):
    assert get_field_values(test.bitfield, test.payload) == test.correct_result



import subprocess


def run_cli_command(cmd_and_args: list[str]):
    result = subprocess.run(cmd_and_args, capture_output=True, text=True)
    return result.returncode, result.stdout


def test_version():
    retcode, output = run_cli_command(['payloadinspector', '--version'])
    assert retcode == 0
    assert output == 'payloadinspector 1.0.1\n'

def test_standard_arguments():
    retcode, output = run_cli_command(['payloadinspector',
                              '-b', "('field1', 4), ('field2', 4), ('field3', 8)",
                              '-p', "0xac 0x11"])
    assert retcode == 0
    assert output == 'field1: 10, field2: 12, field3: 17\n'

def test_standard_arguments_with_comma():
    retcode, output = run_cli_command(['payloadinspector',
                              '-b', "('field1', 4), ('field2', 4), ('field3', 8)",
                              '-p', "0xac, 0x11"])
    assert retcode == 0
    assert output == 'field1: 10, field2: 12, field3: 17\n'

def test_too_many_payload_bytes():
    retcode, output = run_cli_command(['payloadinspector',
                              '-b', "('field1', 4), ('field2', 4), ('field3', 8)",
                              '-p', "0xac 0x11 0x32"])
    assert retcode == 0
    assert output == '''WARNING: Bitfield is 16 bits, and payload is 24 bits. Extra payload data is discarded.
field1: 10, field2: 12, field3: 17\n'''

def test_too_few_payload_bytes():
    retcode, output = run_cli_command(['payloadinspector',
                              '-b', "('field1', 4), ('field2', 4), ('field3', 8)",
                              '-p', "0xac"])
    assert retcode == 1
    assert output == 'ERROR: Bitfield is 16 bits, but payload is only 8 bits.\n'

def test_standard_arguments_binary():
    retcode, output = run_cli_command(['payloadinspector',
                              '-b', "('field1', 4), ('field2', 4), ('field3', 8)",
                              '-p', "0xac, 0x11",
                              '-f', 'bin'])
    assert retcode == 0
    assert output == 'field1: 0b1010, field2: 0b1100, field3: 0b10001\n'

def test_standard_arguments_single_element():
    retcode, output = run_cli_command(['payloadinspector',
                              '-b', "('field1', 4), ('field2', 4), ('field3', 8)",
                              '-p', "0xac, 0x11",
                              '-f', 'field2'])
    assert retcode == 0
    assert output == 'field2: 12\n'

def test_standard_arguments_format_and_element_selection():
    retcode, output = run_cli_command(['payloadinspector',
                              '-b', "('field1', 4), ('field2', 4), ('field3', 8)",
                              '-p', "0xac, 0x11",
                              '-f', 'field1:bin, field3'])
    assert retcode == 0
    assert output == 'field1: 0b1010, field3: 17\n'

def test_bitfield_from_struct():
    retcode, output = run_cli_command(['payloadinspector',
                              '-s', 'tests/header.h:message'])
    assert retcode == 0
    assert output == '''field1: 1
field2: 2
big_value: 32
field3: 3\n'''



from dataclasses import dataclass
from typing import Iterator
import re

@dataclass
class BitFieldElement:
    name: str
    n_bits: int

    def __str__(self) -> str:
        return f'{self.name}: {self.n_bits}'

class BitField:
    def __init__(self, bitfield_elements: list) -> None:
        self._data = []
        for element in bitfield_elements:
            if isinstance(element, BitFieldElement):
                self._data.append(element)
            elif isinstance(element, tuple) and list(map(type, element)) == [str, int]:
                self._data.append(BitFieldElement(*element))
            else:
                raise TypeError(f'Unparsable bitfield element: {element}. Format is (\'name\', n_bits)')
            if self._data[-1].name in [e.name for e in self._data[:-1]]:
                raise ValueError(f'Bitfield element name occurs several times: {element[0]}')

    def __str__(self) -> str:
        return '\n'.join([str(e) for e in self._data])

    def __iter__(self) -> Iterator[BitFieldElement]:
        return self._data.__iter__()

    def __getitem__(self, key) -> BitFieldElement:
        return self._data.__getitem__(key)

    def __eq__(self, other):
        return len(self) == len(other) and \
               all([self[i] == other[i] for i in range(len(self))])

    def __len__(self) -> int:
        """Returns the number of elements in the BitField"""
        return self._data.__len__()

    @property
    def total_bits(self) -> int:
        """Total number of bits in BitField"""
        return sum([e.n_bits for e in self._data])

    @classmethod
    def from_string(cls, string: str):
        """
        Create a BitField object from a string.

        Args:
            string: String describing the format of the BitField as a list of tuples.
        """
        result = []
        current_tuple = []
        string = string.replace(' ', '').replace('\n', '').replace('[', '').replace(']', '')
        for token in string.split(','):
            if '(' in token:
                field_name = token.replace('(', '').replace(')', '').replace('\'', '').replace('"', '')
                current_tuple.append(field_name)
            else:
                n_bits = token.replace('(', '').replace(')', '')
                current_tuple.append(int(n_bits))
            if ')' in token:
                result.append(tuple(current_tuple))
                current_tuple = []
        return cls(result)

    @classmethod
    def from_c_source(cls, file_name: str, struct_name: str):
        """
        Create a BitField object based on a C struct bitfield.

        The struct must have a bit size for all elements.
        The data type and its size are ignored.

            Args:
                file_name: Filename of a C source file.
                struct_name: Name of the struct in the file to load.

            Returns:
                BitField: BitField object modeled after the C struct.
        """
        struct_text = cls._get_struct_text(file_name, struct_name)
        struct_elements = struct_text.split(':')
        elements = []
        for i in range(len(struct_elements) - 1):
            name = struct_elements[i].split()[-1]
            n_bits_string = struct_elements[i+1].split()[0].replace(';', '')
            try:
                n_bits = int(n_bits_string)
            except ValueError:
                raise ValueError(f'Could not parse bit size {n_bits_string}')
            elements.append((name, n_bits))
        if len(elements) < struct_text.count(';') - 1:
            raise ValueError('Some elements are missing bit size')
        return cls(elements)

    @classmethod
    def _get_struct_text(cls, file_name: str, struct_name: str) -> str:
        """
        Extract a given C struct from a source/header file.

            Args:
                file_name: Filename of a C source file.
                struct_name: Name of the struct in the file to load.

            Returns:
                str: The struct definition as a string.
        """
        with open(file_name, 'r') as file:
            content = file.read()
        # Try to find the struct defined in the format
        # struct <struct_name>
        # {
        #   ...
        # };
        struct_text = re.search(f'struct\\s*{struct_name}\\s*'+'{[^{]*}[^;]*;', content)
        if not struct_text:
            # Try to find the struct defined in the format
            # typedef struct
            # {
            #   ...
            # } <struct_name>;
            struct_text = re.search('{[^{]*}[^;]*'+f'{struct_name};', content)
        if not struct_text:
            raise ValueError(f'Could not find struct {struct_name}')
        struct_text = struct_text.group()
        return struct_text

class Payload:
    def __init__(self, data: list[int]) -> None:
        for i, value in enumerate(data):
            if not isinstance(value, int):
                raise TypeError(f'Payload value {i}: {value} is not an integer')
            if value < 0 or value > 255:
                raise ValueError(f'Fayload value {i}: {value} outside legal range [0, 255]')
        self._data = data

    def __iter__(self) -> Iterator[int]:
        return self._data.__iter__()

    def __getitem__(self, key: int) -> int:
        return self._data.__getitem__(key)

    def __eq__(self, other):
        return self._data == other._data

    def __len__(self) -> int:
        return self._data.__len__()

    @property
    def total_bits(self) -> int:
        return self.__len__() * 8

    @classmethod
    def from_string(cls, string: str):
        # Remove [',', '[', ']', ' ']
        string = string.replace(',', '').replace('[', '').replace(']', '').replace(' ', '')

        # Insert whitespaces at the correct position
        if string.startswith('0x'):
            string = ' '.join((string[i:i+4] for i in range(0, len(string), 4)))
        else:
            string = ' '.join((string[i:i+2] for i in range(0, len(string), 2)))

        return cls([int(i, 16) for i in string.split()])

def get_bits_from_byte(byte, start_bit: int, n_bits: int) -> int:
    """
    Extract a series of bits from a byte
    """
    return (byte >> (8-start_bit-n_bits)) & ((1 << n_bits) - 1)

def format_output(result: dict[str, int], format=''):
    output_string = ''
    # Create list of (name, encoding) tuples
    if format == '' or format == 'dec':
        elements = [(name, 'dec') for name in result.keys()]
    elif format == 'hex' or format == 'bin':
        elements = [(name, format) for name in result.keys()]
    else:
        elements = []
        element_formats = format.split(',')
        for e in element_formats:
            components = e.split(':')
            name = components[0].replace(' ', '')
            encoding = 'dec' if len(components) == 1 else components[1].replace(' ', '')
            elements.append((name, encoding))

    # Add each element with correct encoding
    for e in elements:
        element_name = e[0]
        encoding = e[1]
        if element_name not in result.keys():
            raise ValueError(f'Element name in format string does not exist in bitfield: {element_name}')
        if encoding == 'dec':
            output_string += f'{element_name}: {result[element_name]}, '
        elif encoding == 'hex':
            output_string += f'{element_name}: {hex(result[element_name])}, '
        elif encoding == 'bin':
            output_string += f'{element_name}: {bin(result[element_name])}, '
    return output_string.rstrip(', ')


def get_field_values(bit_field: BitField, payload: Payload) -> dict[str, int]:
    """
    Find the value of each element in a BitField.
    """
    values = {}
    current_byte = 0
    bits_left_in_current_byte = 8
    for field in bit_field:
        current_field_value = 0
        unread_bits_in_current_field = field.n_bits
        while unread_bits_in_current_field > 0:
            bits_to_read = min(bits_left_in_current_byte, unread_bits_in_current_field)
            temp_field_value = get_bits_from_byte(payload[current_byte], 8-bits_left_in_current_byte, bits_to_read)
            unread_bits_in_current_field -= bits_to_read
            bits_left_in_current_byte -= bits_to_read
            current_field_value += temp_field_value << unread_bits_in_current_field
            if bits_left_in_current_byte == 0:
                current_byte += 1
                bits_left_in_current_byte = 8
        values[field.name] = current_field_value
    return values



import pytest
from payloadinspector import Payload

def test_init_with_ints():
    data = [10, 128, 255]
    payload = Payload(data)
    assert all(payload[i] == data[i] for i in range(3))

def test_init_with_hex():
    data = [0x0a, 0x80, 0xff]
    payload = Payload(data)
    assert all(payload[i] == data[i] for i in range(3))

def test_init_with_string():
    data = [0x0a, 'value', 0xff]
    with pytest.raises(TypeError):
        payload = Payload(data)

def test_init_out_of_range():
    data = [0x0a, 300, 0xff]
    with pytest.raises(ValueError):
        payload = Payload(data)

def test_iteration():
    data = [0x0a, 0x80, 0xff]
    payload = Payload(data)
    assert [val for val in payload] == data

def test_equal():
    data1 = [10, 128, 255]
    payload1 = Payload(data1)
    data2 = [0x0a, 0x80, 0xff]
    payload2 = Payload(data2)
    assert payload1 == payload2

def test_unequal_size():
    data1 = [10, 128, 255]
    payload1 = Payload(data1)
    data2 = [0x0a, 0x80]
    payload2 = Payload(data2)
    assert payload1 != payload2

def test_unequal_value():
    data1 = [10, 128, 255]
    payload1 = Payload(data1)
    data2 = [0x0a, 0x80, 0x00]
    payload2 = Payload(data2)
    assert payload1 != payload2

def test_len():
    data = [0x0a, 0x80, 0xff]
    payload = Payload(data)
    assert len(payload) == 3

def test_total_bits():
    data = [0x0a, 0x80, 0xff]
    payload = Payload(data)
    assert payload.total_bits == 24

def string_initializer_test(string):
    data = [0x0a, 0x80, 0xff]
    payload_from_list = Payload(data)
    payload_from_string = Payload.from_string(string)

    assert payload_from_list == payload_from_string

def test_string_initialization_0x():
    string = "0x0a 0x80 0xff"
    string_initializer_test(string)

def test_string_initialization_no_0x():
    string = "0a 80 ff"
    string_initializer_test(string)

def test_string_initialization_0x_with_comma():
    string = "0x0a, 0x80, 0xff"
    string_initializer_test(string)

def test_string_initialization_with_comma():
    string = "0a, 80, ff"
    string_initializer_test(string)

def test_string_initialization_0x_with_comma_and_square_brackets():
    string = "[0x0a, 0x80, 0xff]"
    string_initializer_test(string)

def test_string_initialization_with_comma_and_square_brackets():
    string = "[0a, 80, ff]"
    string_initializer_test(string)

def test_string_initialization_0x_no_whitespace():
    string = "0x0a0x800xff"
    string_initializer_test(string)

def test_string_initialization_no_0x_no_whitespace():
    string = "0a80ff"
    string_initializer_test(string)
